import { Options } from "./options";
import { Dictionary } from "./dictionary";

interface Combination {
    word : string;
    left : number
}

interface SubWord {
    word : string;
    position : number;
}

class Main {
    elements : { [key: string]: any } = {};
    dictionaries : Dictionary[] = [];
    currentDictionary : Dictionary;
    cachedWords : { [key: string]: string } = {};

    constructor() {
        this.assignElements();
        
        Options.dictionaries.forEach((dict, id) => {
            this.elements.opt_dict.innerHTML += `<option value="${id}">${dict.name}</option>`;
            this.dictionaries.push(new Dictionary(dict.url, dict.name, dict.nonWords));
        });
        this.currentDictionary = this.dictionaries[0];

        this.elements.opt_dict.addEventListener('change', () => {
            this.cachedWords = {};
            this.currentDictionary = this.dictionaries[this.elements.opt_dict.selectedIndex];
            this.translate();
        });

        this.elements.text_input.addEventListener('input', () => { 
            this.translate();
        });
    }

    assignElements() {
        Options.elements.forEach(elementName => {
            const element = document.getElementById(elementName);
            if(!element) {
                console.error(`Element ${elementName} not found!`);
                return;
            }
            this.elements[elementName] = element;
        });
    }

    findSubwords(word : string) : SubWord[] {
        let subWords : SubWord[] = [];
        const allWords = Object.keys(this.currentDictionary.words);
        for(let i = 0; i < allWords.length; i++) {
            let pos = word.indexOf(allWords[i]);
            if(pos != -1) {
                subWords.push({ word: allWords[i], position: pos });
            }
        }
        return subWords;
    }

    getCombination(word : string, subWords : SubWord[], startingIndex : number = 0) : Combination {
        let combinations : Combination[] = [];
        
        for(let i = 0; i < word.length; i++) {
            let atLeft = word.substring(0, i);
            subWords.forEach(subWord => {
                if(subWord.position != i+startingIndex) return;
                let translatedSubWord = this.currentDictionary.words[subWord.word];
                
                let furtherComb = this.getCombination(word.substring(i + subWord.word.length), subWords, i+startingIndex+subWord.word.length);
                combinations.push({
                    word: atLeft + translatedSubWord + furtherComb.word, 
                    left: i + furtherComb.left 
                });
            });
        }

        combinations.push({
            word: word,
            left: word.length
        })

        return this.findBestCombination(combinations);
    }

    translateWord(word : string) {
        if(!word) return '';
        if(this.currentDictionary.nonWords.includes(word)) return '';

        let cachedWord = this.cachedWords[word];
        if(cachedWord) return cachedWord + ' ';

        let subWords = this.findSubwords(word);
        let translated = this.getCombination(word, subWords).word;
        this.cachedWords[word] = translated;

        return translated + ' ';
    }

    findBestCombination(combinations : Combination[]) : Combination {
        let minLeft = Math.min(...combinations.map(comb => comb.left));
        combinations = combinations.filter(comb => comb.left == minLeft);
        let minLength = Math.min(...combinations.map(comb => comb.word.length));
        combinations = combinations.filter(comb => comb.word.length == minLength);
        return combinations[0];
    }

    translate() {
        if(!this.currentDictionary  || !this.currentDictionary.loaded) {
            console.log('Dictionary not loaded!')
            return;
        }

        let inputText = this.elements.text_input.value.trim();
        inputText = inputText.toLowerCase();

        if(inputText == '') {
            this.cachedWords = {};
        }
        
        let inputWords : string[] = inputText.split(' ');
        let outputWords = inputWords.map(word => this.translateWord(word));
        this.elements.text_output.value = outputWords.join('');
        
    }

        
}

const main : Main = new Main();